@@ -2,7 +2,7 @@
                           syntaxreader.cpp  -  description
                              -------------------
     begin                : Wed Nov 28 2001
-    copyright            : (C) 2001-2018 by Andre Simon
+    copyright            : (C) 2001-2021 by Andre Simon
     email                : a.simon@mailbox.org
  ***************************************************************************/
 
@@ -27,6 +27,7 @@ along with Highlight.  If not, see <http://www.gnu.org/licenses/>.
 
 #include "syntaxreader.h"
 #include "stringtools.h"
+#include "keystore.h"
 #include "enums.h"
 
 namespace highlight
@@ -64,8 +65,10 @@ SyntaxReader::SyntaxReader() :
     allowNestedComments ( true ),
     reformatCode ( false ),
     assertEqualLength(false),
+    paramsNeedUpdate(false),
     rawStringPrefix(0),
     continuationChar(0),
+    keywordCount(0),
     validateStateChangeFct(NULL),
     decorateFct(NULL),
     decorateLineBeginFct(NULL),
@@ -99,7 +102,7 @@ void  SyntaxReader::initLuaState(Diluculum::LuaState& ls, const string& langDefP
     string::size_type Pos = langDefPath.find_last_of ( Platform::pathSeparator );
     ls["HL_LANG_DIR"] =langDefPath.substr ( 0, Pos+1 );
 
-    ls["HL_INPUT_FILE"] = ls["HL_PLUGIN_PARAM"] = pluginParameter;
+    ls["HL_PLUGIN_PARAM"] = pluginParameter;
     ls["HL_OUTPUT"] = type;
 
     ls["Identifiers"]=REGEX_IDENTIFIER;
@@ -171,6 +174,8 @@ LoadResult SyntaxReader::load ( const string& langDefPath, const string& pluginR
         lua_register (ls.getState(), "AddKeyword", luaAddKeyword);
         lua_register (ls.getState(), "RemoveKeyword", luaRemoveKeyword);
         lua_register (ls.getState(), "AddPersistentState", luaAddPersistentState);
+        lua_register (ls.getState(), "OverrideParam", luaOverrideParam);
+        lua_register (ls.getState(), "StoreValue", KeyStore::luaStore);
 
         SyntaxReader **s = (SyntaxReader **)lua_newuserdata(ls.getState(), sizeof(SyntaxReader *));
         *s=this;
@@ -180,12 +185,12 @@ LoadResult SyntaxReader::load ( const string& langDefPath, const string& pluginR
         ls.doFile (langDefPath);
 
         langDesc = ls["Description"].value().asString();
-        
+
         if (ls["Categories"].value() !=Diluculum::Nil){
 
             Diluculum::LuaValueMap categoryMap;
             categoryMap = ls["Categories"].value().asTable();
-                
+
             for(Diluculum::LuaValueMap::const_iterator it = categoryMap.begin(); it != categoryMap.end(); ++it)
             {
                 categories.append(it->second.asString());
@@ -213,7 +218,7 @@ LoadResult SyntaxReader::load ( const string& langDefPath, const string& pluginR
         int kwId=0;
         while (ls["Keywords"][idx].value() !=Diluculum::Nil) {
             kwId= generateNewKWClass ( ls["Keywords"][idx]["Id"].value().asInteger() );
-            
+
             if (ls["Keywords"][idx]["List"].value()!=Diluculum::Nil) {
                 int listIdx=1;
                 Diluculum::LuaVariable luaList=ls["Keywords"][idx]["List"];
@@ -232,11 +237,7 @@ LoadResult SyntaxReader::load ( const string& langDefPath, const string& pluginR
                 if (ls["Keywords"][idx]["Priority"].value()!=Diluculum::Nil) {
                     priority=ls["Keywords"][idx]["Priority"].value().asInteger();
                 }
-                
-                //string semantics;
-                //if (ls["Keywords"][idx]["Semantics"].value()!=Diluculum::Nil)
-                //         constraintFilename =ls["Keywords"][idx]["Semantics"].value().asString();
-                
+
                 unsigned int constraintLineNum=0;
                 string constraintFilename, semanticDescription;
                 if (ls["Keywords"][idx]["Constraints"].value()!=Diluculum::Nil) {
@@ -250,22 +251,14 @@ LoadResult SyntaxReader::load ( const string& langDefPath, const string& pluginR
             }
             idx++;
         }
-        
-        if (globals.count("GeneratorOverride")) {
-            idx=1;
-            Diluculum::LuaValue lVal = ls["GeneratorOverride"][idx].value();
-            while (lVal !=Diluculum::Nil) {
-                pluginConfigOverride[StringTools::change_case(lVal["Param"].asString())] = lVal["Value"].asString();
-                idx++;
-                lVal = ls["GeneratorOverride"][idx].value();
-            }
-        }
-        
+
+        keywordCount = kwId;
+
         if (globals.count("KeywordFormatHints")) {
             idx=1;
             Diluculum::LuaValue lVal = ls["KeywordFormatHints"][idx].value();
             while (lVal !=Diluculum::Nil) {
-                
+
                 int kwStyleOverride = lVal["Id"].asInteger();
 
                 if (lVal["Bold"]!=Diluculum::Nil){
@@ -277,13 +270,13 @@ LoadResult SyntaxReader::load ( const string& langDefPath, const string& pluginR
                 if (lVal["Underline"]!=Diluculum::Nil){
                     kwStyleOverride += lVal["Underline"].asBoolean() ? 512 : 4096;
                 }
-                 
+
                 overrideStyles.push_back(kwStyleOverride);
                 idx++;
                 lVal = ls["KeywordFormatHints"][idx].value();
             }
         }
-        
+
         if (globals.count("Comments")) {
 
             int listIdx=1;
@@ -406,20 +399,20 @@ LoadResult SyntaxReader::load ( const string& langDefPath, const string& pluginR
                 string lang= ls["NestedSections"][listIdx]["Lang"].value().asString();
                 string openDelim=StringTools::trim(ls["NestedSections"][listIdx]["Delimiter"][1].value().asString());
                 regex.insert(regex.begin(), 1, new RegexElement(EMBEDDED_CODE_BEGIN, EMBEDDED_CODE_BEGIN, openDelim, 0, -1, lang));
-                
+
                 string closeDelim=StringTools::trim(ls["NestedSections"][listIdx]["Delimiter"][2].value().asString());
                 nestedStateEndDelimiters[getNewPath(lang)] = closeDelim;
-                
+
                 bool allowInnerSectionsFlag=true;
                 if (ls["NestedSections"][listIdx]["Sealed"].value()!=Diluculum::Nil){
                     allowInnerSectionsFlag = !ls["NestedSections"][listIdx]["Sealed"].value().asBoolean();
                 }
                 allowInnerSections[getNewPath(lang)] = allowInnerSectionsFlag;
-                
+
                 ++listIdx;
             }
             //allow host syntax
-            allowInnerSections[getCurrentPath()] = true;               
+            allowInnerSections[getCurrentPath()] = true;
         }
 
         if (globals.count("HeaderInjection")) {
@@ -433,7 +426,7 @@ LoadResult SyntaxReader::load ( const string& langDefPath, const string& pluginR
         if (globals.count("EncodingHint")) {
             encodingHint = ls["EncodingHint"].value().asString();
         }
-        
+
         // load hook functions
         if (globals.count("OnStateChange")) {
             validateStateChangeFct=new Diluculum::LuaFunction(ls["OnStateChange"].value().asFunction());
@@ -478,7 +471,7 @@ int SyntaxReader::getKeywordListGroup ( const string &s ) {
     }
     return 0;
 }
-    
+
 int SyntaxReader::luaAddKeyword (lua_State *L)
 {
     int retVal=0;
@@ -512,6 +505,31 @@ int SyntaxReader::luaRemoveKeyword (lua_State *L)
     return 1;
 }
 
+
+void SyntaxReader::overrideParam(const string& name, const string& val)
+{
+    pluginConfigOverride[name] = val;
+    paramsNeedUpdate=true;
+}
+
+
+int SyntaxReader::luaOverrideParam (lua_State *L)
+{
+    int retVal=0;
+    if (lua_gettop(L)==2) {
+        const char* name = lua_tostring(L, 1);
+        const char* val = lua_tostring(L, 2);
+        lua_getglobal(L, GLOBAL_SR_INSTANCE_NAME);
+        SyntaxReader **a=reinterpret_cast<SyntaxReader **>(lua_touserdata(L, 3));
+        if (*a) {
+            (*a)->overrideParam(name, val);
+            retVal=1;
+        }
+    }
+    lua_pushboolean(L, retVal);
+    return 1;
+}
+
 void SyntaxReader::restoreLangEndDelim(const string& langPath)
 {
     if ( !langPath.empty()&& nestedStateEndDelimiters.count(langPath) ) {
@@ -524,18 +542,21 @@ bool SyntaxReader::allowsInnerSection(const string& langPath)
     return allowInnerSections[langPath];
 }
 
-unsigned int SyntaxReader::generateNewKWClass ( int classID )
+unsigned int SyntaxReader::generateNewKWClass ( int classID, const char *prefix )
 {
     char className[5]= {0};
-    snprintf(className, sizeof(className), "kw%c", ('a'+classID-1));
+    snprintf(className, sizeof(className), "%s%c", prefix, 'a'+classID-1);
 
     unsigned int newClassID=0;
     bool found=false;
+
     while (!keywordClasses.empty() && newClassID<keywordClasses.size() && !found ) {
+
         found = ( className==keywordClasses.at(newClassID++) );
     }
     if ( !found ) {
         newClassID++;
+
         keywordClasses.push_back ( className );
     }
     return newClassID;
@@ -590,11 +611,11 @@ bool SyntaxReader::matchesOpenDelimiter ( const string& token, State s, int open
 void SyntaxReader::addPersistentKeyword(unsigned int groupID, const string& kw){
     ostringstream expr;
     expr <<"AddKeyword(\""<<kw<<"\", "<<groupID<<")";
-             
+
     persistentSnippets.push_back(expr.str());
     persistentSyntaxDescriptions.insert(langDesc);
 }
-    
+
 void SyntaxReader::addPersistentStateRange(unsigned int groupID, unsigned int column,unsigned int length, unsigned int lineNumber, const string& fileName){
     ostringstream expr;
     expr <<"table.insert(Keywords,\n"
@@ -606,7 +627,7 @@ void SyntaxReader::addPersistentStateRange(unsigned int groupID, unsigned int co
          <<"    Filename = [=["<<fileName<<"]=],\n"
          <<" }\n"
          <<"})";
-                
+
     persistentSnippets.push_back(expr.str());
     persistentSyntaxDescriptions.insert(langDesc);
 }
@@ -614,17 +635,17 @@ void SyntaxReader::addPersistentStateRange(unsigned int groupID, unsigned int co
 string SyntaxReader::getPersistentHookConditions() {
     ostringstream expr;
     expr << "ValidDesc = {";
-    
+
     for (auto desc: persistentSyntaxDescriptions) {
         expr << "\"" << desc << "\",";
     }
-    
+
     expr << "}\nif ValidDesc[desc] ~= nil then return end\n\n";
-    
+
     return expr.str();
-    
+
 }
- 
+
 int SyntaxReader::luaAddPersistentState (lua_State *L)
 {
     int retVal=0;
@@ -638,7 +659,7 @@ int SyntaxReader::luaAddPersistentState (lua_State *L)
                 (*a)->addKeyword(kwgroupID, keyword);
                 (*a)->addPersistentKeyword(kwgroupID, keyword);
             }
-            
+
             retVal=1;
         }
     }
@@ -656,16 +677,25 @@ int SyntaxReader::luaAddPersistentState (lua_State *L)
     }
     lua_pushboolean(L, retVal);
     return 1;
-} 
+}
 
 bool SyntaxReader::requiresTwoPassRun(){
     return persistentSyntaxDescriptions.count(langDesc)>0;
 }
 
+bool SyntaxReader::requiresParamUpdate(){
+    bool ret = paramsNeedUpdate;
+    paramsNeedUpdate = false;
+    return ret;
+}
+
 void SyntaxReader::clearPersistentSnippets() {
     persistentSyntaxDescriptions.clear();
     persistentSnippets.clear();
 }
 
+int SyntaxReader::getKeywordCount() const {
+    return keywordCount;
+}
 
 }
