@@ -2,7 +2,7 @@
                                mainwindow.h
                              -------------------
     begin                : Mo 16.03.2009
-    copyright            : (C) 2009-2020 by Andre Simon
+    copyright            : (C) 2009-2021 by Andre Simon
     email                : a.simon@mailbox.org
  ***************************************************************************/
 
@@ -56,12 +56,16 @@ along with Highlight.  If not, see <http://www.gnu.org/licenses/>.
 #include <QTextStream>
 #include <QFileSystemWatcher>
 #include <QDesktopServices>
+#include <QStyleFactory>
 
 #include "version.h"
 #include "codegenerator.h"
 #include "htmlgenerator.h"
+#include "lspprofile.h"
+#include "lspclient.h"
 
 #include <map>
+#include <vector>
 #include "enums.h"
 
 typedef multimap<string, string> MMap;
@@ -85,10 +89,19 @@ private:
     Ui::MainWindowClass *ui;
     MMap assocByExtension, assocByFilename;
     SMap assocByShebang, rememberedAssoc, encodingHints;
+    std::map<std::string, highlight::LSPProfile> lspProfiles;
+
+
     QString fileOpenFilter;
     QString savedClipboardContent;
     QString twoPassOutFile;
 
+    std::string lsProfile;
+    std::string lsExecutable;               ///< server executable path
+    std::string lsSyntax;                   ///< language definition which can be enhanced using the LS
+    int lsDelay;
+    std::vector<std::string> lsOptions; ///< server executable start options
+
     QShortcut *copyShortcut;
     QShortcut *pasteShortcut;
     QFileSystemWatcher scriptWatcher;
@@ -110,7 +123,7 @@ private:
     QString getDistThemePath();
     QString getDistLangPath(const string & suffix);
     QString getDistPluginPath();
-    QString getDistFileConfigPath();
+    QString getDistFileConfigPath(QString name);
     QString getDistFileFilterPath();
 
     QString getWindowsShortPath(const QString & path);
@@ -120,7 +133,12 @@ private:
 
     void selectSingleFile(QLineEdit*, const QString&, const QString&);
     bool loadFileTypeConfig();
+    bool loadLSProfiles();
+
+    bool initializeLS(highlight::CodeGenerator * generator, bool tellMe);
+
     bool shortNamesDisabled();
+
     void highlight2Clipboard(bool getDataFromCP);
 
     string analyzeFile(const string& file);
@@ -166,6 +184,11 @@ private slots:
     void on_pbSVGChooseStyleIncFile_clicked();
     void on_pbCopyAndPaste_clicked();
 
+    void on_pbLSInitialize_clicked();
+    void on_leLSExec_textChanged();
+    void on_pbSelWorkspace_clicked();
+    void on_pbSelExecutable_clicked();
+
     void on_action_License_triggered();
     void on_action_Changelog_triggered();
     void on_action_Plug_Ins_triggered();
@@ -177,6 +200,7 @@ private slots:
     void plausibility();
     void updatePreview();
     void openFiles();
+    void loadLSProfile();
 
     void on_actionDock_floating_panels_toggled(bool arg1);
     void on_pbPluginReadFilePath_clicked();
