@@ -2,7 +2,7 @@
                           datadir.h  -  description
                              -------------------
     begin                : Sam March 1 2003
-    copyright            : (C) 2003-2019 by Andre Simon
+    copyright            : (C) 2003-2021 by Andre Simon
     email                : a.simon@mailbox.org
  ***************************************************************************/
 
@@ -33,10 +33,10 @@ along with Highlight.  If not, see <http://www.gnu.org/licenses/>.
 #include <Diluculum/LuaState.hpp>
 
 #include "stringtools.h"
+#include "lspprofile.h"
 
-using namespace std;
 
-typedef map<string, string> StringMap;
+typedef std::map<std::string, std::string> StringMap;
 
 
 /** \brief Manages access to installation directories.
@@ -48,102 +48,111 @@ typedef map<string, string> StringMap;
 
 class DataDir
 {
-    vector <string> possibleDirs;
-    
+    std::vector <std::string> possibleDirs;
+
 private:
 
-    const string searchFile(const string path);
-    
-    string getFileBaseName(const string& fileName);
+    const std::string searchFile(const std::string path);
+
+    std::string getFileBaseName(const std::string& fileName);
+
+    std::string analyzeFile ( const std::string& file );
+
+    void readLuaList(const std::string& paramName, const std::string& langName,Diluculum::LuaValue &luaVal, StringMap* extMap);
 
-    string analyzeFile ( const string& file );
-    
-    void readLuaList(const string& paramName, const string& langName,Diluculum::LuaValue &luaVal, StringMap* extMap);
-    
     // object needs to life until cin is no longer used
-    stringstream cin_bufcopy;
-    
+    std::stringstream cin_bufcopy;
+
+    std::map<std::string, highlight::LSPProfile> lspProfiles;
+
 public:
 
     StringMap assocByExtension, assocByFilename, assocByShebang, encodingHint;
 
-    static string LSB_DATA_DIR;
-    static string LSB_CFG_DIR;
-    static string LSB_DOC_DIR;
+    static std::string LSB_DATA_DIR;
+    static std::string LSB_CFG_DIR;
+    static std::string LSB_DOC_DIR;
 
     /** search for a valid installation directory
         \param  userDefinedDir Directory defined by user
         \return True if directory was found */
-    void initSearchDirectories ( const string &userDefinedDir );
-
-    // Deprecated
-    void searchDataDir( const string &userDefinedDir );
+    void initSearchDirectories ( const std::string &userDefinedDir );
 
     const void printConfigPaths();
 
     /**  \param file file
          \return Location of given syntax definition */
-    const string getLangPath ( const string & file) ;
+    const std::string getLangPath ( const std::string & file) ;
 
     /** \return System themes location */
-    const string getThemePath ( ) ;
+    const std::string getThemePath () ;
 
     /** \return System syntax location */
-    const string getLangPath ( ) ;
+    const std::string getLangPath () ;
 
     /** \return System syntax location */
-    const string getSystemDataPath ( ) ;
+    const std::string getSystemDataPath () ;
 
     /** \return Location of plugins */
-    const string getPluginPath ();
+    const std::string getPluginPath ();
 
     /** \param file file
      *  \param base16 set to true if the theme is located in the base16 sub dir
         \return Location of given theme */
-    const string getThemePath ( const string & file, bool base16=false ) ;
+    const std::string getThemePath ( const std::string & file, bool base16=false ) ;
 
     /** \param file file
         \return Location of given configuration file */
-    const string getFiletypesConfPath (const string &path="filetypes.conf");
-    
+    const std::string getFiletypesConfPath (const std::string &path="filetypes.conf");
+
     /** \param file file
         \return Location of given plugin file */
-    const string getPluginPath (const string &);
+    const std::string getPluginPath (const std::string &);
 
     /** \return Location of GUI menu translation files */
-    const string getI18nDir();
+    const std::string getI18nDir();
 
     /** \return Location of GUI file extension filter files */
-    const string getExtDir();
+    const std::string getExtDir();
 
     /** \return Location of documentation (README) files (GUI) */
-    const string getDocDir();
+    const std::string getDocDir();
 
     /** \param syntax name
         \return Encoding hint assigned to the syntax */
-    const string getEncodingHint (const string &);
+    const std::string getEncodingHint (const std::string &);
 
     /**
-     * \param fileName input file name 
+     * \param fileName input file name
      * \return file extension or the base filename if no file extension exists
     */
-    string getFileSuffix ( const string &fileName );
+    std::string getFileSuffix ( const std::string &fileName );
 
     /**
      * \param suffix file name suffix
      * \param inputFile name of the input file
      * \param useUserSuffix true if user has defined a syntax suffix
-     * \param forceShebangCheckStdin true if stdin should be peeked to look for shebang patterns 
+     * \param forceShebangCheckStdin true if stdin should be peeked to look for shebang patterns
      * \return file type deferred from extension or file shebang comment
     */
-    string guessFileType ( const string &suffix, const string &inputFile, 
+    std::string guessFileType ( const std::string &suffix, const std::string &inputFile,
                            bool useUserSuffix=false, bool forceShebangCheckStdin=false );
-    
+
     /**
-     * \param name absolute path of filetypes.conf
+     * \param name name of filetypes.conf
      * \return true if successful
     */
-    bool loadFileTypeConfig ( const string& name);
+    bool loadFileTypeConfig ( const std::string& name);
+
+    /**
+     * \param name name of lsp.conf
+     * \return true if successful
+     */
+    bool loadLSPConfig ( const std::string& name);
+
+    bool profileExists(const string &profile);
+
+    highlight::LSPProfile &getProfile(const string &profile);
 };
 
 #endif
