@@ -2,8 +2,8 @@
                           htmlgenerator.h  -  description
                              -------------------
     begin                : Wed Nov 28 2001
-    copyright            : (C) 2001-2015 by Andre Simon
-    email                : andre.simon1@gmx.de
+    copyright            : (C) 2001-2021 by Andre Simon
+    email                : a.simon@mailbox.org
  ***************************************************************************/
 
 
@@ -154,7 +154,7 @@ private:
 
     /** add Lua state variables specific to the output format */
    // virtual void addSpecificVars();
-    
+
     /** Print document header
     */
     string getHeader();
@@ -178,7 +178,7 @@ private:
     /** \return escaped character*/
     virtual string maskCharacter ( unsigned char );
 
-    /** test if anchors should be appied to line numbers*/
+    /** test if anchors should be applied to line numbers*/
     bool attachAnchors;
 
     /**Optional anchor prefix */
@@ -199,6 +199,10 @@ private:
 
     /** @return Newline string */
     string getNewLine();
+
+    virtual string getHoverTagOpen(const string & hoverText);
+
+    virtual string getHoverTagClose();
 };
 
 }
